from __future__ import annotations

from typing import Optional
from pathlib import Path
import subprocess
import sys
import os

import cv2
import numpy as np

from .data_classes import RawImage
from .simple_lama_bindings import SimpleLama
# https://github.com/okaris/simple-lama/tree/main

def blackout(raw_image: RawImage) -> np.ndarray:
    image = raw_image.get_image()

    for box in raw_image.bounding_boxes:
        cv2.rectangle(image, box, (0, 0, 0), -1)

    return image


def get_mask(raw_image: RawImage) -> np.ndarray:
    mask = np.zeros(raw_image.image.shape[:2], dtype=np.uint8)
    for (x, y, w, h) in raw_image.bounding_boxes:
        mask[y:y+h, x:x+w] = 255

    return mask


def quick_impaint(raw_image: RawImage, image: Optional[np.ndarray] = None) -> np.ndarray:
    image = image if image is not None else raw_image.get_image()
    
    mask = get_mask(raw_image)

    # Apply inpainting using the Telea method
    return cv2.inpaint(image, mask, inpaintRadius=3, flags=cv2.INPAINT_TELEA)


def do_generative_impaint(raw_image: RawImage, image: Optional[np.ndarray] = None) -> np.ndarray:
    image = image if image is not None else raw_image.get_image()
    mask = get_mask(raw_image)

    lama = SimpleLama()
    return lama(image=image, mask=mask)


def pixelate_regions(raw_image: RawImage, image: Optional[np.ndarray] = None, pixel_size: int = 10) -> np.ndarray:
    image = image.copy() if image is not None else raw_image.get_image().copy()

    for (x, y, w, h) in raw_image.bounding_boxes:
        roi = image[y:y+h, x:x+w]

        # Resize down and then back up
        temp = cv2.resize(roi, (max(1, w // pixel_size), max(1, h // pixel_size)), interpolation=cv2.INTER_LINEAR)
        pixelated = cv2.resize(temp, (w, h), interpolation=cv2.INTER_NEAREST)

        image[y:y+h, x:x+w] = pixelated

    return image


def pixelate(to_detect: str, generative_impaint: bool = True, debug_drawings: bool = False):
    raw_image = RawImage(to_detect)

    step_dir = raw_image.get_dir("steps")
    def write_image(image: np.ndarray, name: str):
        nonlocal debug_drawings
        
        f = str(step_dir / (name + raw_image.file.suffix))

        if debug_drawings:
            for box in raw_image.bounding_boxes:
                cv2.rectangle(image, box, (0, 255, 255), 1)
        
        cv2.imwrite(f, image)

    write_image(raw_image.image, "step_0")

    step_1 = blackout(raw_image)
    write_image(step_1, "step_1")

    if generative_impaint:
        step_2 = do_generative_impaint(raw_image, image=step_1)
        step_2_alt = quick_impaint(raw_image, image=step_1)
    else:
        step_2 = quick_impaint(raw_image, image=step_1)
        step_2_alt = do_generative_impaint(raw_image, image=step_1)
    write_image(step_2, "step_2")
    write_image(step_2_alt, "step_2_alt")

    step_3 = pixelate_regions(raw_image, image=step_2)
    write_image(step_3, "step_3")
